from django.contrib import admin

from django_rubble.models.number_models import NamedSerialNumber


@admin.register(NamedSerialNumber)
class NamedSerialNumberAdmin(admin.ModelAdmin):
    list_display = ["name", "content_type", "next_counter"]
    list_editable = ["next_counter"]



from django.db import models
from django.utils.translation import gettext_lazy as _

from django_rubble.forms.db_forms import (
    SimplePercentageField as SimplePercentageFormField,
)


class SimplePercentageField(models.DecimalField):
    """Enter and display percentages out of 100 but store them out of 1
    in db as decimals

    Because this is based on `models.DecimalField`, `decimal_places` applies to what is
    stored in the db (/1), not what is shown or typed in (/100). With that said, add two
    (2) to whatever is desired in the form for proper validation.
    """

    description = _(
        "percentage (max {max_digits} digits; {decimal_places} decimal places"
    )
    log_name = "models.PercentageField"

    def __init__(
        self,
        verbose_name=None,
        name=None,
        max_digits=None,
        decimal_places=None,
        **kwargs,
    ):
        if decimal_places is not None:
            self.humanize_decimal_places = int(decimal_places) - 2
        else:
            self.humanize_decimal_places = None

        kwargs.update(
            {
                "max_digits": max_digits,
                "decimal_places": decimal_places,
            }
        )

        super().__init__(verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {"form_class": SimplePercentageFormField}
        kwargs.update(
            decimal_places=self.decimal_places - 2,
        )
        defaults.update(kwargs)

        return super().formfield(**defaults)



from decimal import Decimal

import pytest

from django_rubble.utils.numbers import is_number, trim_trailing_zeros


@pytest.mark.parametrize(
    ("number", "expected"),
    [
        (10, True),
        (3.14, True),
        ("42", True),
        ("3.14", True),
        ("hello", False),
        (None, False),
    ],
)
def test_is_number(number, expected):
    assert is_number(number) == expected


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        (10, Decimal("10")),
        (3.14, Decimal("3.14")),
        ("42", Decimal("42")),
        ("3.1400", Decimal("3.14")),
    ],
)
def test_trim_trailing_zeros(value, expected):
    assert trim_trailing_zeros(value) == expected
    assert Decimal(str(value)).normalize() == expected
